-- nature (Minetest 0.4 mod)
-- Nature is slowly capturing the world!
-- Licenced under the GPLv3

local ABM_DELAY = 7200

local TREE_GROW_DELAY = ABM_DELAY -- 2 hrs
local TREE_GROW_CHANCE = 5 -- * 0.5%
local DENSITY = 3 -- allow 3 trunks in one 'cross'
local MINIMUM_LIGHT = 8 -- light needed for the trees to grow

local CACTUS_GROW_DELAY = ABM_DELAY -- seconds
local CACTUS_GROW_CHANCE = 6 -- * 0.5%

local PAPYRUS_GROW_DELAY = ABM_DELAY -- seconds
local PAPYRUS_GROW_CHANCE = 15 -- * 0.5%

local ABM_CHANCE = 40

local BLOSSOM_DELAY = ABM_DELAY
local APPLE_DELAY = ABM_DELAY
local BLOSSOM_ABM_CHANCE = 100
local BLOSSOM_CHANCE = 5 -- * 0.5%

-- Randomization
math.randomseed(os.time())

-- NODES

-- Blossom node
minetest.register_node("nature:blossom", {
    description = "Blossom",
    drawtype = "allfaces_optional",
    tile_images = { "nature_blossom.png" },
    --inventory_image = minetest.inventorycube("nature_blossom.png"),
    paramtype = "light",
    material = minetest.digprop_leaveslike(1.0),
    --furnace_burntime = 1,
})

-- Irontree node
minetest.register_node("nature:irontree", {
    description = "Irontree",
    tile_images = {"nature_irontree_top.png", "nature_irontree_top.png", "nature_irontree.png"},
    --inventory_image = minetest.inventorycube("nature_irontree_top.png", "nature_irontree.png", "nature_irontree.png"),
    is_ground_content = true,
    material = minetest.digprop_woodlike(1.0),
    --cookresult_itemstring = 'default:iron_lump 1',
    --furnace_burntime = 30,
})

-- CRAFTS

-- Irontree
minetest.register_craft({
    output = 'nature:irontree 1',
    recipe = {
        { 'default:iron_lump 1' },
        { 'default:tree 1' },
    }
})

minetest.register_craft({
    type = "fuel",
    recipe = "nature:irontree",
    burntime = 30,
})

minetest.register_craft({
    type = "cooking",
    output = "default:iron_lump",
    recipe = "nature:irontree",
    cooktime = 4,
})

-- Blossom

minetest.register_craft({
    type = "fuel",
    recipe = "nature:blossom",
    burntime = 1,
})

-- ABMs

-- Tree growing
minetest.register_abm({
    nodenames = { "default:leaves" },
    interval = TREE_GROW_DELAY,
    chance = ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > TREE_GROW_CHANCE) then
            return
        end

        if(minetest.env:get_node_light(pos, nil) < MINIMUM_LIGHT) then
            return
        end
        
        local trunk_count = 0
        local jungle_trunk_count = 0
        local iron_trunk_count = 0
        
        -- Check for trunks in area
        for i = -1, 1 do
        for j = -1, 0 do
        for k = -1, 1 do
            local current_node = {
                x = pos.x + i,
                y = pos.y + j,
                z = pos.z + k
            }
            if(math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                if(minetest.env:get_node(current_node).name == "default:tree") then
                trunk_count = trunk_count + 1
                elseif(minetest.env:get_node(current_node).name ==
                "default:jungletree") then
                    jungle_trunk_count = jungle_trunk_count + 1
                elseif(minetest.env:get_node(current_node).name ==
                "nature:irontree") then
                    iron_trunk_count = iron_trunk_count + 1
                end
            end
        end
        end
        end

        local all_trunks = trunk_count + jungle_trunk_count + iron_trunk_count
        
        -- If there is at least 1 trunk and there are not many of them...
        if (all_trunks > 0) and (all_trunks < DENSITY) then
            minetest.env:remove_node(pos)
            if(math.random(1, all_trunks) <= trunk_count) then
                minetest.env:add_node(pos, {name = "default:tree"})
            elseif(math.random(1, all_trunks) <= jungle_trunk_count) then
                minetest.env:add_node(pos, {name = "default:jungletree"})
            else
                minetest.env:add_node(pos, {name = "nature:irontree"})
            end
            print ('[nature] A trunk has grown at (' .. pos.x .. ',' .. pos.y .. ',' .. pos.z .. ')')
            for i = -1, 1 do
            for j = -1, 1 do
            for k = -1, 1 do
                local current_node = {
                    x = pos.x + i,
                    y = pos.y + j,
                    z = pos.z + k
                }
                if(minetest.env:get_node(current_node).name == "air") then
                    minetest.env:add_node(current_node, {name = "default:leaves"})
                end
            end
            end
            end
        end
    end
})


-- Cactus growing
minetest.register_abm({
    nodenames = { "default:cactus" },
    interval = CACTUS_GROW_DELAY,
    chance = ABM_CHANCE,
    
    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > CACTUS_GROW_CHANCE) then
            return
        end
        -- Grow with a chance (choosing which side to grow)
        for i = -1, 1 do
        for j = 0, 1 do
        for k = -1, 1 do
            if(math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                if(math.random(1, 30) == 1) then
                    local new_pos = {
                        x = pos.x + i,
                        y = pos.y + j,
                        z = pos.z + k
                    }
                    if(minetest.env:get_node(new_pos).name == "air") then
                        minetest.env:add_node(new_pos, {name = "default:cactus"})
                        print ('[nature] A cactus has grown at (' .. new_pos.x .. ',' .. new_pos.y .. ',' .. new_pos.z .. ')')
                    end
                end
            end
        end
        end
        end
    end
})

-- Papyrus growing
minetest.register_abm({
    nodenames = { "default:papyrus" },
    interval = PAPYRUS_GROW_DELAY,
    CHANCE = ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > PAPYRUS_GROW_CHANCE) then
            return
        end
        
        if(minetest.env:get_node_light(pos, nil) < MINIMUM_LIGHT) then
            return
        end
        
        local under = {
            x = pos.x,
            y = pos.y - 1,
            z = pos.z
        }
        -- Grow sides if connected to land from bottom
        if(minetest.env:get_node(under).name == "default:grass")
        or (minetest.env:get_node(under).name == "default:dirt") then
            for i = -1, 1 do
            for j = -1, 1 do
                if(math.abs(i) + math.abs(j) == 1) then
                    if(math.random(1, 30) == 1) then
                        local new_pos = {
                            x = pos.x + i,
                            y = pos.y,
                            z = pos.z + j
                        }
                        if(minetest.env:get_node(new_pos).name == "air") then
                            minetest.env:add_node(new_pos, {name = "default:papyrus"})
                            print('[nature] A papyrus has grown at (' .. new_pos.x .. ',' .. new_pos.y .. ',' .. new_pos.z .. ')')
                        end
                    end
                end
            end
            end
        end
        -- Grow up
        local above = {
            x = pos.x,
            y = pos.y + 1,
            z = pos.z
        }
        if(minetest.env:get_node(above).name == "air") then
            minetest.env:add_node(above, {name = "default:papyrus"})
            print('[nature] A papyrus has grown at (' .. above.x .. ',' .. above.y .. ',' .. above.z .. ')')
        end
    end
})

-- Blossoming
minetest.register_abm({
    nodenames = { "default:leaves" },
    interval = BLOSSOM_DELAY,
    chance = BLOSSOM_ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        if(math.random(1, 1000) > BLOSSOM_CHANCE) then
            return
        end
        minetest.env:remove_node(pos)
        minetest.env:add_node(pos, { name = "nature:blossom" })
    end
})

-- Apples growing
minetest.register_abm({
    nodenames = { "nature:blossom" },
    interval = APPLE_DELAY,
    chance = BLOSSOM_ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        if(math.random(1, 1000) > BLOSSOM_CHANCE) then
            return
        end
        minetest.env:remove_node(pos)
        minetest.env:add_node(pos, { name = "default:apple" })
    end
})

-- Apples decaying
minetest.register_abm({
    nodenames = { "default:apple" },
    interval = APPLE_DELAY,
    chance = BLOSSOM_ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        if(math.random(1, 1000) > BLOSSOM_CHANCE) then
            return
        end
        minetest.env:remove_node(pos)
        minetest.env:add_node(pos, { name = "default:leaves" })
    end
})
